@@ -248,6 +248,13 @@ class _CodeFragment:
         fpo.write("\n}  //extern C\n")
         fpo.write("\n")
         return self.funcname
+        
+    def _sanitize_filename(self, file):
+        if file.find("\\\\") >= 0: # already converted
+            return file
+        if file.find("\\") >= 0:
+            return file.replace("\\", "\\\\")
+        return file
 
     def _clean_up_import_line(self, line):
         line = re.sub(r'\&\s+', ' &', line)
@@ -276,8 +283,10 @@ class _CodeFragment:
         subline = lineno-len(source)-1
         
         importspec = False
+
+        basefile = self._sanitize_filename(filename)
         
-        self.prefunc.append('#line %d "%s"' % (subline, filename))
+        self.prefunc.append('#line %d "%s"' % (subline, basefile))
         for line in source:
             subline += 1
             try:
@@ -315,9 +324,9 @@ class _CodeFragment:
                 self.functions[funcx] = (rettype, funcname, args)
                 cargstr = ", ".join(map(self.type2c, args))
                 rettype = self.type2c(rettype)
+                self.sourcecode.append("#line %d" % subline)
                 self.arguments.append("void*")
                 self.variables.append("func_ptr_%s" % funcname)
-                self.sourcecode.append("#line %d" % subline)
                 self.sourcecode.append("""
                         %s (*%s)(%s);
                         %s = (%s (*)(%s))%s;
@@ -436,8 +445,8 @@ def _get_caller_info(levels=1):
     fr = inspect.currentframe()
     for i in range(levels+1):
         fr  = fr.f_back
-    (file, lineno, function, code_context, index) = inspect.getframeinfo(fr)
-    return (fr, file, lineno)
+    (filename, lineno, function, code_context, index) = inspect.getframeinfo(fr)
+    return (fr, filename, lineno)
         
 def C(source):
     """Compile at runtime and run code in-line"""
@@ -463,14 +472,14 @@ def embed_c(source):
     """Compile and load C functions; return CDLL object"""
     return _embed_or_inline_c(source, False)
 
-def _embed_or_inline_c(source, inline, file=None, lineno=0):
+def _embed_or_inline_c(source, inline, filename=None, lineno=0):
     """Compile and load C code, but don't execute; return CDLL object"""
     code = _CodeFragment()
     comp = _EmbedCompile()
     sourcecode = source.split("\n")
-    if file is None:
-        (fr, file, lineno) = _get_caller_info(2)
-    code.parse_embed_code(file, lineno+1, source.split("\n"), inline)
+    if filename is None:
+        (fr, filename, lineno) = _get_caller_info(2)
+    code.parse_embed_code(filename, lineno+1, sourcecode, inline)
     fr = inspect.currentframe().f_back.f_back
     _import_all_vars(code, fr)
     (dll, func) = comp.temp_compile(code)
@@ -501,16 +510,25 @@ def _load_func_parse(sourcepath):
         processed[sourcepath] = True
 
 _ustringtype = None
+_ispython3 = None
+
+def _is_3():
+    # test python 2.6 & 3.0
+    global _ustringtype
+    global _ispython3
+    if _ustringtype is None:
+        _ispython3 = False
+        try:
+            _ustringtype = unicode  # fails on 3.0+
+            _ispython3 = False
+        except NameError:
+            _ustringtype = type('')
+            _ispython3 = True
+    return _ispython3
         
 def _isunicode(text):
     # test python 2.6 & 3.0
-    global _ustringtype
-    if _ustringtype is None:
-        try:
-            _ustringtype = unicode  # fails on 3.0+
-        except NameError:
-            _ustringtype = type('')
-        
+    _is_3()
     if type(text) is _ustringtype:
         return True
     else:
@@ -667,10 +685,7 @@ def _get_dll_name(file):
     return "%s.cpp.%s" % (file, dllext)
 
 def _convert_to_ascii(list):
-    # test 3.0 constructs; if 3.0+, then we're ok
-    try:
-        _ustringtype = unicode  # fails on 3.0+
-    except NameError:
+    if _is_3(): # 3.0+ is already unicde, so ignore
         return list
     # for other version, rencode
     rlist = []
